var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/monad/option.ts
var Some = (value) => value;
var None = () => void 0;
var present = (value) => value !== void 0;
var empty = (value) => value === void 0;

// src/monad/either.ts
var Left = (value) => {
  return {
    tag: "left",
    left: value
  };
};
var Right = (value) => {
  return {
    tag: "right",
    right: value
  };
};
var isLeft = (value) => value.tag === "left";
var isRight = (value) => value.tag === "right";
var fold = (value, left, right) => {
  switch (value.tag) {
    case "left":
      return left(value.left);
    case "right":
      return right(value.right);
  }
};

// src/computation/lazy.ts
var Lazy = (callback) => callback;
var AsyncLazy = (callback) => callback;
var computed = (value) => typeof value !== "function";
var compute = (value) => computed(value) ? value : value();

// src/error/typed.ts
var TypedError = (type, cause, stack) => {
  return {
    type,
    cause,
    stack
  };
};
var isTypedError = (value, type) => value.type === type;
var wrap = (value) => {
  const internal = new Error(value.cause);
  internal.name = value.type;
  internal.stack = value.stack;
  return internal;
};
var unwrap = (value) => {
  return {
    cause: value.message,
    type: value.name,
    stack: value.stack
  };
};

// src/error/unknown.ts
var UnknownError = (cause, stack) => {
  if (typeof cause === "string") {
    return TypedError("unknown-error", cause, stack);
  }
  const internal = safeUnknownError(cause);
  return TypedError(
    "unknown-error",
    internal.message,
    internal.message
  );
};
var safeUnknownError = (error) => error instanceof Error ? error : new Error(`${error}`);

// src/error/call.ts
var safeThrowCall = (call, onError) => {
  var _a;
  try {
    return call();
  } catch (error) {
    const internal = safeUnknownError(error);
    return Left((_a = onError == null ? void 0 : onError.call(void 0, internal)) != null ? _a : UnknownError(internal));
  }
};
var safeAsyncThrowCall = (call, onError) => __async(void 0, null, function* () {
  var _a;
  try {
    return yield call();
  } catch (error) {
    const internal = safeUnknownError(error);
    return Left((_a = onError == null ? void 0 : onError.call(void 0, internal)) != null ? _a : UnknownError(internal));
  }
});

// src/error/io.ts
var ReadError = (cause, stack) => TypedError("read-error", cause, stack);
var WriteError = (cause, stack) => TypedError("write-error", cause, stack);
var isReadError = (value) => isTypedError(value, "read-error");
var isWriteError = (value) => isTypedError(value, "write-error");
export {
  AsyncLazy,
  Lazy,
  Left,
  None,
  ReadError,
  Right,
  Some,
  TypedError,
  UnknownError,
  WriteError,
  compute,
  computed,
  empty,
  fold,
  isLeft,
  isReadError,
  isRight,
  isTypedError,
  isWriteError,
  present,
  safeAsyncThrowCall,
  safeThrowCall,
  safeUnknownError,
  unwrap,
  wrap
};
//# sourceMappingURL=index.js.map